@@ -14,11 +14,42 @@
 #include <sstream>
 #include <stdexcept>
 #include <string>
+#include <iostream>
 
 #include "xtensor_config.hpp"
+#include <xtl/xsequence.hpp>
+#include <xtl/xspan_impl.hpp>
+
+#ifdef __GNUC__
+#define XTENSOR_UNUSED_VARIABLE __attribute__ ((unused))
+#else
+#define XTENSOR_UNUSED_VARIABLE
+#endif
 
 namespace xt
 {
+    struct missing_type {};
+    namespace {
+        missing_type XTENSOR_UNUSED_VARIABLE missing;
+    }
+
+    namespace detail
+    {
+        template <class... Args>
+        struct last_type_is_missing_impl
+            : std::is_same<missing_type, xtl::mpl::back_t<xtl::mpl::vector<Args...>>>
+        {
+        };
+
+        template <>
+        struct last_type_is_missing_impl<>
+            : std::false_type
+        {
+        };
+
+        template <class... Args>
+        constexpr bool last_type_is_missing = last_type_is_missing_impl<Args...>::value;
+    }
 
     /*******************
      * broadcast_error *
@@ -147,6 +178,11 @@ namespace xt
         {
         }
 
+        template <class S, std::size_t dim>
+        inline void check_index_impl(const S&, missing_type)
+        {
+        }
+
         template <class S, std::size_t dim, class T, class... Args>
         inline void check_index_impl(const S& shape, T arg, Args... args)
         {
@@ -165,6 +201,11 @@ namespace xt
     {
     }
 
+    template <class S>
+    inline void check_index(const S&, missing_type)
+    {
+    }
+
     template <class S, class Arg, class... Args>
     inline void check_index(const S& shape, Arg arg, Args... args)
     {
@@ -178,6 +219,11 @@ namespace xt
             // Too many arguments: drop the first
             check_index(shape, args...);
         }
+        else if (detail::last_type_is_missing<Args...>)
+        {
+            // Too few arguments & last argument xt::missing: postfix index with zeros
+            detail::check_index_impl<S, 0>(shape, arg, args...);
+        }
         else
         {
             // Too few arguments: ignore the beginning of the shape
@@ -194,7 +240,7 @@ namespace xt
         auto dst = static_cast<size_type>(last - first);
         It efirst = last - static_cast<std::ptrdiff_t>((std::min)(shape.size(), dst));
         std::size_t axis = 0;
-        
+
         while (efirst != last)
         {
             if (*efirst >= value_type(shape[axis]) && shape[axis] != 1)
