@@ -423,12 +423,12 @@ namespace xt
     namespace detail
     {
         template <class C, class = void_t<>>
-        struct has_storage_iterator : std::false_type
+        struct has_linear_iterator : std::false_type
         {
         };
 
         template <class C>
-        struct has_storage_iterator<C, void_t<decltype(std::declval<C>().storage_cbegin())>>
+        struct has_linear_iterator<C, void_t<decltype(std::declval<C>().linear_cbegin())>>
             : std::true_type
         {
         };
@@ -437,9 +437,9 @@ namespace xt
     template <class C>
     XTENSOR_CONSTEXPR_RETURN auto linear_begin(C& c) noexcept
     {
-        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
+        return xtl::mpl::static_if<detail::has_linear_iterator<C>::value>([&](auto self)
         {
-            return self(c).storage_begin();
+            return self(c).linear_begin();
         }, /*else*/ [&](auto self)
         {
             return self(c).begin();
@@ -449,9 +449,9 @@ namespace xt
     template <class C>
     XTENSOR_CONSTEXPR_RETURN auto linear_end(C& c) noexcept
     {
-        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
+        return xtl::mpl::static_if<detail::has_linear_iterator<C>::value>([&](auto self)
         {
-            return self(c).storage_end();
+            return self(c).linear_end();
         }, /*else*/ [&](auto self)
         {
             return self(c).end();
@@ -461,9 +461,9 @@ namespace xt
     template <class C>
     XTENSOR_CONSTEXPR_RETURN auto linear_begin(const C& c) noexcept
     {
-        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
+        return xtl::mpl::static_if<detail::has_linear_iterator<C>::value>([&](auto self)
         {
-            return self(c).storage_cbegin();
+            return self(c).linear_cbegin();
         }, /*else*/ [&](auto self)
         {
             return self(c).cbegin();
@@ -473,9 +473,9 @@ namespace xt
     template <class C>
     XTENSOR_CONSTEXPR_RETURN auto linear_end(const C& c) noexcept
     {
-        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
+        return xtl::mpl::static_if<detail::has_linear_iterator<C>::value>([&](auto self)
         {
-            return self(c).storage_cend();
+            return self(c).linear_cend();
         }, /*else*/ [&](auto self)
         {
             return self(c).cend();
