@@ -7,9 +7,11 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#include "gtest/gtest.h"
+#include "test_common.hpp"
 #include "test_xsemantic.hpp"
 
+TEST_SUITE_BEGIN("adaptor_semantic");
+
 namespace xt
 {
     using vector_type = std::vector<int>;
@@ -35,64 +37,55 @@ namespace xt
     using get_test_adaptor_t = typename get_test_adaptor<C>::type;
 
     template <class C>
-    class adaptor_semantic : public ::testing::Test
+    class adaptor_semantic
     {
     public:
         using container_type = C;
         using adaptor_type = get_test_adaptor_t<C>;
     };
 
-    using testing_types = ::testing::Types<xarray_dynamic, xtensor_dynamic>;
-    TYPED_TEST_SUITE(adaptor_semantic, testing_types);
+    #define ADAPTOR_SEMANTIC_TYPES xarray_dynamic, xtensor_dynamic
 
-    TYPED_TEST(adaptor_semantic, xsimd_info)
+    TEST_CASE_TEMPLATE("xsimd_info", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
 #if defined(XTENSOR_USE_XSIMD)
         std::cout << "Built with XSIMD" << std::endl;
-    #if defined(XSIMD_X86_INSTR_SET)
-        std::cout << "Using X86 Instruction set: " << XSIMD_INSTR_SET << std::endl;
-    #elif defined(XSIMD_X86_AMD_INSTR_SET)
-        std::cout << "Using AMD Instruction set: " << XSIMD_INSTR_SET << std::endl;
-    #elif defined(XSIMD_ARM_INSTR_SET)
-        std::cout << "Using ARM Instruction set: " << XSIMD_INSTR_SET << std::endl;
-    #else
-        std::cout << "Using unknown Instruction set: " << XSIMD_INSTR_SET << std::endl;
-    #endif
+        std::cout << " arch "<<xsimd::default_arch::name() << std::endl;
 #else
         std::cout << "Built without XSIMD" << std::endl;
 #endif
     }
 
-    TYPED_TEST(adaptor_semantic, a_plus_b)
+    TEST_CASE_TEMPLATE("a_plus_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::plus<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
+        SUBCASE("row_major + row_major")
         {
-            SCOPED_TRACE("row_major + row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a + tester.ra;
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major + column_major")
         {
-            SCOPED_TRACE("row_major + column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a + tester.ca;
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major + central_major")
         {
-            SCOPED_TRACE("row_major + central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a + tester.cta;
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major + unit_major")
         {
-            SCOPED_TRACE("row_major + unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a + tester.ua;
@@ -100,37 +93,37 @@ namespace xt
         }
     }
 
-    TYPED_TEST(adaptor_semantic, a_minus_b)
+    TEST_CASE_TEMPLATE("a_minus_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::minus<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
 
+        SUBCASE("row_major - row_major")
         {
-            SCOPED_TRACE("row_major - row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a - tester.ra;
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major - column_major")
         {
-            SCOPED_TRACE("row_major - column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a - tester.ca;
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major - central_major")
         {
-            SCOPED_TRACE("row_major - central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a - tester.cta;
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major - unit_major")
         {
-            SCOPED_TRACE("row_major - unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a - tester.ua;
@@ -138,37 +131,37 @@ namespace xt
         }
     }
 
-    TYPED_TEST(adaptor_semantic, a_times_b)
+    TEST_CASE_TEMPLATE("a_times_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::multiplies<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
 
+        SUBCASE("row_major * row_major")
         {
-            SCOPED_TRACE("row_major * row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a * tester.ra;
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major * column_major")
         {
-            SCOPED_TRACE("row_major * column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a * tester.ca;
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major * central_major")
         {
-            SCOPED_TRACE("row_major * central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a * tester.cta;
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major * unit_major")
         {
-            SCOPED_TRACE("row_major * unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a * tester.ua;
@@ -176,37 +169,37 @@ namespace xt
         }
     }
 
-    TYPED_TEST(adaptor_semantic, a_divide_by_b)
+    TEST_CASE_TEMPLATE("a_divide_by_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::divides<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
 
+        SUBCASE("row_major / row_major")
         {
-            SCOPED_TRACE("row_major / row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a / tester.ra;
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major / column_major")
         {
-            SCOPED_TRACE("row_major / column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a / tester.ca;
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major / central_major")
         {
-            SCOPED_TRACE("row_major / central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a / tester.cta;
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major / unit_major")
         {
-            SCOPED_TRACE("row_major / unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a / tester.ua;
@@ -214,13 +207,13 @@ namespace xt
         }
     }
 
-    TYPED_TEST(adaptor_semantic, a_plus_equal_b)
+    TEST_CASE_TEMPLATE("a_plus_equal_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::plus<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
 
+        SUBCASE("row_major += row_major")
         {
-            SCOPED_TRACE("row_major += row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -228,8 +221,8 @@ namespace xt
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major += column_major")
         {
-            SCOPED_TRACE("row_major += column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -237,8 +230,8 @@ namespace xt
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major += central_major")
         {
-            SCOPED_TRACE("row_major += central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -246,8 +239,8 @@ namespace xt
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major += unit_major")
         {
-            SCOPED_TRACE("row_major += unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -256,13 +249,13 @@ namespace xt
         }
     }
 
-    TYPED_TEST(adaptor_semantic, a_minus_equal_b)
+    TEST_CASE_TEMPLATE("a_minus_equal_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::minus<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
 
+        SUBCASE("row_major -= row_major")
         {
-            SCOPED_TRACE("row_major -= row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -270,8 +263,8 @@ namespace xt
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major -= column_major")
         {
-            SCOPED_TRACE("row_major -= column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -279,8 +272,8 @@ namespace xt
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major -= central_major")
         {
-            SCOPED_TRACE("row_major -= central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -288,8 +281,8 @@ namespace xt
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major -= unit_major")
         {
-            SCOPED_TRACE("row_major -= unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -298,13 +291,13 @@ namespace xt
         }
     }
 
-    TYPED_TEST(adaptor_semantic, a_times_equal_b)
+    TEST_CASE_TEMPLATE("a_times_equal_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::multiplies<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
 
+        SUBCASE("row_major *= row_major")
         {
-            SCOPED_TRACE("row_major *= row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -312,8 +305,8 @@ namespace xt
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major *= column_major")
         {
-            SCOPED_TRACE("row_major *= column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -321,8 +314,8 @@ namespace xt
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major *= central_major")
         {
-            SCOPED_TRACE("row_major *= central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -330,8 +323,8 @@ namespace xt
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major *= unit_major")
         {
-            SCOPED_TRACE("row_major *= unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -340,13 +333,13 @@ namespace xt
         }
     }
 
-    TYPED_TEST(adaptor_semantic, a_divide_by_equal_b)
+    TEST_CASE_TEMPLATE("a_divide_by_equal_b", TypeParam, ADAPTOR_SEMANTIC_TYPES)
     {
         operation_tester<std::divides<>, TypeParam> tester;
-        using adaptor_type = typename TestFixture::adaptor_type;
+        using adaptor_type = typename adaptor_semantic<TypeParam>::adaptor_type;
 
+        SUBCASE("row_major /= row_major")
         {
-            SCOPED_TRACE("row_major /= row_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -354,8 +347,8 @@ namespace xt
             EXPECT_EQ(tester.res_rr, b);
         }
 
+        SUBCASE("row_major /= column_major")
         {
-            SCOPED_TRACE("row_major /= column_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -363,8 +356,8 @@ namespace xt
             EXPECT_EQ(tester.res_rc, b);
         }
 
+        SUBCASE("row_major /= central_major")
         {
-            SCOPED_TRACE("row_major /= central_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -372,8 +365,8 @@ namespace xt
             EXPECT_EQ(tester.res_rct, b);
         }
 
+        SUBCASE("row_major /= unit_major")
         {
-            SCOPED_TRACE("row_major /= unit_major");
             vector_type v;
             adaptor_type b(v);
             b = tester.a;
@@ -381,4 +374,6 @@ namespace xt
             EXPECT_EQ(tester.res_ru, b);
         }
     }
+    #undef ADAPTOR_SEMANTIC_TYPES
 }
+TEST_SUITE_END();
\ No newline at end of file
