@@ -7,7 +7,7 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#include "gtest/gtest.h"
+
 #include "xtensor/xarray.hpp"
 #include "xtensor/xtensor.hpp"
 #include "xtensor/xmanipulation.hpp"
@@ -21,22 +21,22 @@ namespace xt
 
     TEST(xarray, shaped_constructor)
     {
+        SUBCASE("row_major constructor")
         {
-            SCOPED_TRACE("row_major constructor");
             row_major_result<> rm;
             xarray_dynamic ra(rm.m_shape, layout_type::row_major);
             compare_shape(ra, rm);
         }
 
+        SUBCASE("column_major constructor")
         {
-            SCOPED_TRACE("column_major constructor");
             column_major_result<> cm;
             xarray<int, layout_type::column_major> ca(cm.m_shape);
             compare_shape(ca, cm);
         }
 
+        SUBCASE("from shape")
         {
-            SCOPED_TRACE("from shape");
             std::array<std::size_t, 3> shp = {5, 4, 2};
             std::vector<std::size_t> shp_as_vec = {5, 4, 2};
             auto ca = xarray<int, layout_type::column_major>::from_shape({3, 2, 1});
@@ -56,8 +56,8 @@ namespace xt
 
     TEST(xarray, valued_constructor)
     {
+        SUBCASE("row_major valued constructor")
         {
-            SCOPED_TRACE("row_major valued constructor");
             row_major_result<> rm;
             int value = 2;
             xarray_dynamic ra(rm.m_shape, value, layout_type::row_major);
@@ -66,8 +66,8 @@ namespace xt
             EXPECT_EQ(ra.storage(), vec);
         }
 
+        SUBCASE("column_major valued constructor")
         {
-            SCOPED_TRACE("column_major valued constructor");
             column_major_result<> cm;
             int value = 2;
             xarray<int, layout_type::column_major> ca(cm.m_shape, value);
@@ -100,15 +100,15 @@ namespace xt
         int value = 2;
         xarray_dynamic a(res.m_shape, res.m_strides, value);
 
+        SUBCASE("copy constructor")
         {
-            SCOPED_TRACE("copy constructor");
             xarray_dynamic b(a);
             compare_shape(a, b);
             EXPECT_EQ(a.storage(), b.storage());
         }
 
+        SUBCASE("assignment operator")
         {
-            SCOPED_TRACE("assignment operator");
             row_major_result<> r;
             xarray_dynamic c(r.m_shape, 0);
             EXPECT_NE(a.storage(), c.storage());
@@ -124,16 +124,16 @@ namespace xt
         int value = 2;
         xarray_dynamic a(res.m_shape, res.m_strides, value);
 
+        SUBCASE("move constructor")
         {
-            SCOPED_TRACE("move constructor");
             xarray_dynamic tmp(a);
             xarray_dynamic b(std::move(tmp));
             compare_shape(a, b);
             EXPECT_EQ(a.storage(), b.storage());
         }
 
+        SUBCASE("move assignment")
         {
-            SCOPED_TRACE("move assignment");
             row_major_result<> r;
             xarray_dynamic c(r.m_shape, 0);
             EXPECT_NE(a.storage(), c.storage());
@@ -314,6 +314,19 @@ namespace xt
         EXPECT_EQ(a.layout(), c.layout());
     }
 
+    TEST(xarray, operator_brace)
+    {
+#ifdef XTENSOR_ENABLE_ASSERT
+        xt::xarray<size_t> a = {{0,1,2}, {3,4,5}};
+        EXPECT_THROW(a(2, 0), std::runtime_error);
+        EXPECT_THROW(a(0, 3), std::runtime_error);
+
+        xt::xarray<size_t> b = {{0,1,2}};
+        EXPECT_THROW(a(0, 3), std::runtime_error);
+        EXPECT_THROW(a(1, 3), std::runtime_error);
+#endif
+    }
+
     TEST(xarray, periodic)
     {
         xt::xarray<size_t> a = {{0,1,2}, {3,4,5}};
@@ -336,7 +349,7 @@ namespace xt
         EXPECT_EQ(a.back(), 6);
     }
 
-    TEST(xarray, flat)
+TEST(xarray, flat)
     {
         {
             xt::xarray<size_t, xt::layout_type::row_major> a = {{0,1,2}, {3,4,5}};
@@ -345,6 +358,9 @@ namespace xt
             a.flat(4) = 40;
             a.flat(5) = 50;
             EXPECT_EQ(a, b);
+#ifdef XTENSOR_ENABLE_ASSERT
+            EXPECT_THROW(a.flat(6), std::runtime_error);
+#endif
         }
         {
             xt::xarray<size_t, xt::layout_type::column_major> a = {{0,1,2}, {3,4,5}};
@@ -353,6 +369,9 @@ namespace xt
             a.flat(3) = 40;
             a.flat(5) = 50;
             EXPECT_EQ(a, b);
+#ifdef XTENSOR_ENABLE_ASSERT
+            EXPECT_THROW(a.flat(6), std::runtime_error);
+#endif
         }
     }
 
