@@ -7,7 +7,7 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#include "gtest/gtest.h"
+#include "test_common_macros.hpp"
 
 #include <complex>
 #include <xtl/xcomplex.hpp>
@@ -111,6 +111,14 @@ namespace xt
         EXPECT_EQ(*(it.operator->()), 1.0);
     }
 
+    TEST(xcomplex, arg)
+    {
+        xarray<std::complex<double>> cmplarg_0 = {{0.40101756 + 0.71233018i, 0.62731701 + 0.42786349i, 0.32415089 + 0.2977805i},
+                                                  {0.24475928 + 0.49208478i, 0.69475518 + 0.74029639i, 0.59390240 + 0.35772892i},
+                                                  {0.63179202 + 0.41720995i, 0.44025718 + 0.65472131i, 0.08372648 + 0.37380143i}};
+        xarray<double> res = xt::arg(cmplarg_0);
+
+    }
     TEST(xcomplex, abs_angle_conj)
     {
         xarray<std::complex<double>> cmplarg_0 = {{0.40101756 + 0.71233018i, 0.62731701 + 0.42786349i, 0.32415089 + 0.2977805i},
@@ -195,7 +203,7 @@ namespace xt
         xarray<double> A = {{0.81745298, 0.75933774, 0.44016704},
                             {0.54959488, 1.01524554, 0.69331814},
                             {0.75711643, 0.78897806, 0.38306348}};
-        xarray<double> B = xt::conj(A);
+        xarray<double> B = xt::real(xt::conj(A));
         EXPECT_EQ(A, B);
     }
 
