@@ -7,7 +7,7 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#include "gtest/gtest.h"
+#include "test_common_macros.hpp"
 
 #include "xtensor/xarray.hpp"
 #include "xtensor/xtensor.hpp"
@@ -24,10 +24,6 @@ namespace xt
         using result_type = R;
     };
 
-    using testing_types = ::testing::Types<row_major_result<>, column_major_result<>,
-        central_major_result<>, unit_shape_result<>>;
-    TYPED_TEST_SUITE(xiterator_test, testing_types);
-
     using std::size_t;
 
     template <layout_type L, class R, class S>
@@ -54,38 +50,8 @@ namespace xt
             iter2++;
         }
 
-        EXPECT_EQ(*iter, expected) << "preincrement operator doesn't give expected result";
-        EXPECT_EQ(*iter2, expected) << "postincrement operator doesn't give expected result";
-    }
-
-    TYPED_TEST(xiterator_test, increment)
-    {
-        typename TestFixture::result_type rm;
-        {
-            SCOPED_TRACE("same shape - row_major iterator");
-            test_increment<layout_type::row_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - row_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_increment<layout_type::row_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("same shape - column_major iterator");
-            test_increment<layout_type::column_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - column_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_increment<layout_type::column_major>(rm, rm.shape());
-        }
+        CHECK_MESSAGE(*iter==expected, "preincrement operator doesn't give expected result");
+        CHECK_MESSAGE(*iter2==expected, "postincrement operator doesn't give expected result");
     }
 
     template <layout_type L, class R, class S>
@@ -110,39 +76,12 @@ namespace xt
         iter += nb_inc;
         auto iter3 = iter2 + nb_inc;
 
-        EXPECT_EQ(*iter, expected) << "preincrement operator doesn't give expected result";
-        EXPECT_EQ(*iter3, expected) << "postincrement operator doesn't give expected result";
-        EXPECT_EQ(iter2[nb_inc], expected) << "postincrement operator doesn't give expected result";
-    }
 
-    TYPED_TEST(xiterator_test, random_increment)
-    {
-        typename TestFixture::result_type rm;
-        {
-            SCOPED_TRACE("same shape - row_major iterator");
-            test_random_increment<layout_type::row_major>(rm, rm.shape());
-        }
 
-        {
-            SCOPED_TRACE("broadcasting shape - row_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_random_increment<layout_type::row_major>(rm, rm.shape());
-        }
+        CHECK_MESSAGE(*iter==expected,  "preincrement operator doesn't give expected result");
+        CHECK_MESSAGE(*iter3==expected,  "postincrement operator doesn't give expected result");
+        CHECK_MESSAGE(iter2[nb_inc]==expected,  "postincrement operator doesn't give expected result");
 
-        {
-            SCOPED_TRACE("same shape - column_major iterator");
-            test_random_increment<layout_type::column_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - column_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_random_increment<layout_type::column_major>(rm, rm.shape());
-        }
     }
 
     template <layout_type L, class R, class S>
@@ -161,40 +100,10 @@ namespace xt
             ++iter;
         }
 
-        EXPECT_EQ(iter, last) << "iterator doesn't reach the end";
+        CHECK_MESSAGE(iter==last, "iterator doesn't reach the end");
         EXPECT_FALSE(iter < last);
     }
 
-    TYPED_TEST(xiterator_test, end)
-    {
-        typename TestFixture::result_type rm;
-        {
-            SCOPED_TRACE("same shape - row_major iterator");
-            test_end<layout_type::row_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - row_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_end<layout_type::row_major>(rm, sh);
-        }
-
-        {
-            SCOPED_TRACE("same shape - column_major iterator");
-            test_end<layout_type::column_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - column_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_end<layout_type::column_major>(rm, sh);
-        }
-    }
-
     template <layout_type L, class R, class S>
     void test_decrement(const R& result, const S& shape)
     {
@@ -218,38 +127,8 @@ namespace xt
             ++iter;
             iter2++;
         }
-        EXPECT_EQ(*iter, expected) << "predecrement operator doesn't give expected result";
-        EXPECT_EQ(*iter2, expected) << "postdecrement operator doesn't give expected result";
-    }
-
-    TYPED_TEST(xiterator_test, decrement)
-    {
-        typename TestFixture::result_type rm;
-        {
-            SCOPED_TRACE("same shape - row_major iterator");
-            test_decrement<layout_type::row_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - row_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_decrement<layout_type::row_major>(rm, sh);
-        }
-
-        {
-            SCOPED_TRACE("same shape - column_major iterator");
-            test_decrement<layout_type::column_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - column_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_decrement<layout_type::column_major>(rm, sh);
-        }
+        CHECK_MESSAGE(*iter == expected, "predecrement operator doesn't give expected result");
+        CHECK_MESSAGE(*iter2 == expected, "postdecrement operator doesn't give expected result");
     }
 
     template <layout_type L, class R, class S>
@@ -275,38 +154,8 @@ namespace xt
         iter += nb_inc;
         auto iter3 = iter2 + nb_inc;
 
-        EXPECT_EQ(*iter, expected) << "predecrement operator doesn't give expected result";
-        EXPECT_EQ(*iter3, expected) << "postdecrement operator doesn't give expected result";
-    }
-
-    TYPED_TEST(xiterator_test, random_decrement)
-    {
-        typename TestFixture::result_type rm;
-        {
-            SCOPED_TRACE("same shape - row_major iterator");
-            test_random_decrement<layout_type::row_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - row_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_random_decrement<layout_type::row_major>(rm, sh);
-        }
-
-        {
-            SCOPED_TRACE("same shape - column_major iterator");
-            test_random_decrement<layout_type::column_major>(rm, rm.shape());
-        }
-
-        {
-            SCOPED_TRACE("broadcasting shape - column_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_random_decrement<layout_type::column_major>(rm, sh);
-        }
+        CHECK_MESSAGE(*iter == expected , "predecrement operator doesn't give expected result");
+        CHECK_MESSAGE(*iter3 == expected , "postdecrement operator doesn't give expected result");
     }
 
     template <layout_type L, class R, class S>
@@ -321,47 +170,218 @@ namespace xt
         auto iter = a.template rbegin<L>(shape);
         auto last = a.template rend<L>(shape);
 
-        EXPECT_EQ(*iter, data.back()) << "dereferencing rbegin does not result in last element";
-        EXPECT_EQ(*last, data.front()) << "dereferencing rend does not result in first element";
+        CHECK_MESSAGE(*iter==data.back(), "dereferencing rbegin does not result in last element");
+        CHECK_MESSAGE(*last==data.front(), "dereferencing rend does not result in first element");
 
         for (size_type i = 0; i < size; ++i)
         {
             ++iter;
         }
 
-        EXPECT_EQ(iter, last) << "reverse iterator doesn't reach the end";
+        CHECK_MESSAGE(iter==last, "reverse iterator doesn't reach the end");
     }
 
-    TYPED_TEST(xiterator_test, reverse_end)
+    #define XITERATOR_TEST_TYPES\
+        row_major_result<>,\
+        column_major_result<>,\
+        central_major_result<>,\
+        unit_shape_result<>
+
+    TEST_SUITE("xiterator_test")
     {
-        typename TestFixture::result_type rm;
+        TEST_CASE_TEMPLATE("increment", TypeParam, XITERATOR_TEST_TYPES)
         {
-            SCOPED_TRACE("same shape - row_major iterator");
-            test_rend<layout_type::row_major>(rm, rm.shape());
+            using TestFixture = xiterator_test<TypeParam>;
+            typename TestFixture::result_type rm;
+            SUBCASE("same shape - row_major iterator")
+            {
+                test_increment<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - row_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_increment<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("same shape - column_major iterator")
+            {
+                test_increment<layout_type::column_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - column_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_increment<layout_type::column_major>(rm, rm.shape());
+            }
         }
 
+
+        TEST_CASE_TEMPLATE("random_increment", TypeParam, XITERATOR_TEST_TYPES)
         {
-            SCOPED_TRACE("broadcasting shape - row_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_rend<layout_type::row_major>(rm, sh);
+            using TestFixture = xiterator_test<TypeParam>;
+            typename TestFixture::result_type rm;
+            SUBCASE("same shape - row_major iterator")
+            {
+                test_random_increment<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - row_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_random_increment<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("same shape - column_major iterator")
+            {
+                test_random_increment<layout_type::column_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - column_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_random_increment<layout_type::column_major>(rm, rm.shape());
+            }
         }
 
+
+        TEST_CASE_TEMPLATE("end", TypeParam, XITERATOR_TEST_TYPES)
         {
-            SCOPED_TRACE("same shape - column_major iterator");
-            test_rend<layout_type::column_major>(rm, rm.shape());
+            using TestFixture = xiterator_test<TypeParam>;
+            typename TestFixture::result_type rm;
+            SUBCASE("same shape - row_major iterator")
+            {
+                test_end<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - row_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_end<layout_type::row_major>(rm, sh);
+            }
+
+            SUBCASE("same shape - column_major iterator")
+            {
+                test_end<layout_type::column_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - column_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_end<layout_type::column_major>(rm, sh);
+            }
         }
 
+
+        TEST_CASE_TEMPLATE("decrement", TypeParam, XITERATOR_TEST_TYPES)
         {
-            SCOPED_TRACE("broadcasting shape - column_major iterator");
-            layout_result<>::shape_type sh = rm.shape();
-            sh.insert(sh.begin(), 2);
-            sh.insert(sh.begin(), 4);
-            test_rend<layout_type::column_major>(rm, sh);
+            using TestFixture = xiterator_test<TypeParam>;
+            typename TestFixture::result_type rm;
+            SUBCASE("same shape - row_major iterator")
+            {
+                test_decrement<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - row_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_decrement<layout_type::row_major>(rm, sh);
+            }
+
+            SUBCASE("same shape - column_major iterator")
+            {
+                test_decrement<layout_type::column_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - column_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_decrement<layout_type::column_major>(rm, sh);
+            }
+        }
+
+        TEST_CASE_TEMPLATE("random_decrement", TypeParam, XITERATOR_TEST_TYPES)
+        {
+            using TestFixture = xiterator_test<TypeParam>;
+            typename TestFixture::result_type rm;
+            SUBCASE("same shape - row_major iterator")
+            {
+                test_random_decrement<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - row_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_random_decrement<layout_type::row_major>(rm, sh);
+            }
+
+            SUBCASE("same shape - column_major iterator")
+            {
+                test_random_decrement<layout_type::column_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - column_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_random_decrement<layout_type::column_major>(rm, sh);
+            }
+        }
+
+
+        TEST_CASE_TEMPLATE("reverse_end", TypeParam, XITERATOR_TEST_TYPES)
+        {
+            using TestFixture = xiterator_test<TypeParam>;
+            typename TestFixture::result_type rm;
+            SUBCASE("same shape - row_major iterator")
+            {
+                test_rend<layout_type::row_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - row_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_rend<layout_type::row_major>(rm, sh);
+            }
+
+            SUBCASE("same shape - column_major iterator")
+            {
+                test_rend<layout_type::column_major>(rm, rm.shape());
+            }
+
+            SUBCASE("broadcasting shape - column_major iterator")
+            {
+                layout_result<>::shape_type sh = rm.shape();
+                sh.insert(sh.begin(), 2);
+                sh.insert(sh.begin(), 4);
+                test_rend<layout_type::column_major>(rm, sh);
+            }
         }
     }
 
+
+
     template <layout_type L, class R, class S>
     void test_minus(const R& result, const S& shape)
     {
@@ -383,29 +403,29 @@ namespace xt
 
         auto iter = a.template begin<L>() + nb_inc;
         auto iter2 = a.template begin<L>() + nb_inc2;
-        EXPECT_EQ(iter2 - iter, expected) << "operator- doesn't give expected result";
+        CHECK_MESSAGE(iter2 - iter == expected, "operator- doesn't give expected result");
 
         auto riter = a.template rbegin<L>() + nb_inc;
         auto riter2 = a.template rbegin<L>() + nb_inc2;
-        EXPECT_EQ(riter2 - riter, expected) << "operator- doesn't give expected result";
+        CHECK_MESSAGE(riter2 - riter == expected, "operator- doesn't give expected result");
 
         auto diff = a.template end<L>() - a.template begin<L>();
-        EXPECT_EQ(size_type(diff), a.size());
+        CHECK_EQ(size_type(diff), a.size());
 
         auto rdiff = a.template rend<L>() - a.template rbegin<L>();
-        EXPECT_EQ(size_type(rdiff), a.size());
+        CHECK_EQ(size_type(rdiff), a.size());
     }
 
     TEST(xiterator, row_major_minus)
     {
         row_major_result<> rm;
+        SUBCASE("same shape - row_major iterator")
         {
-            SCOPED_TRACE("same shape - row_major iterator");
             test_minus<layout_type::row_major>(rm, rm.shape());
         }
 
+        SUBCASE("broadcasting shape - row_major iterator")
         {
-            SCOPED_TRACE("broadcasting shape - row_major iterator");
             layout_result<>::shape_type sh = rm.shape();
             sh.insert(sh.begin(), 2);
             sh.insert(sh.begin(), 4);
@@ -416,13 +436,13 @@ namespace xt
     TEST(xiterator, column_major_minus)
     {
         column_major_result<> rm;
+        SUBCASE("same shape - column_major iterator")
         {
-            SCOPED_TRACE("same shape - column_major iterator");
             test_minus<layout_type::column_major>(rm, rm.shape());
         }
 
+        SUBCASE("broadcasting shape - column_major iterator")
         {
-            SCOPED_TRACE("broadcasting shape - column_major iterator");
             layout_result<>::shape_type sh = rm.shape();
             sh.insert(sh.begin(), 2);
             sh.insert(sh.begin(), 4);
@@ -490,15 +510,15 @@ namespace xt
         using vector_type = row_major_result<>::vector_type;
         xarray_adaptor<vector_type, layout_type::dynamic> a(rm.storage(), rm.shape(), rm.strides());
         
+        SUBCASE("row_major iterator")
         {
-            SCOPED_TRACE("row_major iterator");
             xarray<vector_type::value_type> dst(a.shape(), 1);
             std::copy(a.cbegin<layout_type::row_major>(), a.cend<layout_type::row_major>(), dst.begin<layout_type::row_major>());
             EXPECT_EQ(a, dst);
         }
 
+        SUBCASE("column_major iterator")
         {
-            SCOPED_TRACE("column_major iterator");
             xarray<vector_type::value_type> dst(a.shape(), 1);
             std::copy(a.cbegin<layout_type::column_major>(), a.cend<layout_type::column_major>(), dst.begin<layout_type::column_major>());
             EXPECT_EQ(a, dst);
@@ -511,15 +531,15 @@ namespace xt
         using vector_type = row_major_result<>::vector_type;
         xarray_adaptor<vector_type, layout_type::dynamic> a(rm.storage(), rm.shape(), rm.strides());
 
+        SUBCASE("row_major iterator")
         {
-            SCOPED_TRACE("row_major iterator");
             xarray<vector_type::value_type> dst(a.shape(), 1);
             std::copy(a.crbegin<layout_type::row_major>(), a.crend<layout_type::row_major>(), dst.rbegin<layout_type::row_major>());
             EXPECT_EQ(a, dst);
         }
 
+        SUBCASE("column_major iterator")
         {
-            SCOPED_TRACE("column_major iterator");
             xarray<vector_type::value_type> dst(a.shape(), 1);
             std::copy(a.crbegin<layout_type::column_major>(), a.crend<layout_type::column_major>(), dst.rbegin<layout_type::column_major>());
             EXPECT_EQ(a, dst);
