@@ -10,7 +10,7 @@
 #include <complex>
 #include <limits>
 
-#include "gtest/gtest.h"
+#include "test_common_macros.hpp"
 #include "xtensor/xarray.hpp"
 #include "xtensor/xadapt.hpp"
 #include "xtensor/xoptional_assembly.hpp"
@@ -330,8 +330,8 @@ namespace xt
         using array_type = xarray<double>;
         array_type a = { {1.2, 2.3}, {3.4, 4.5} };
 
+        SUBCASE("unary function")
         {
-            SCOPED_TRACE("unary function");
             auto fexp = exp(a);
             using assign_traits = xassign_traits<array_type, decltype(fexp)>;
 #if XTENSOR_USE_XSIMD
@@ -345,8 +345,8 @@ namespace xt
 #endif
         }
 
+        SUBCASE("binary function")
         {
-            SCOPED_TRACE("binary function");
             auto fpow = pow(a, a);
             using assign_traits = xassign_traits<array_type, decltype(fpow)>;
 #if XTENSOR_USE_XSIMD
@@ -360,8 +360,8 @@ namespace xt
 #endif
         }
 
+        SUBCASE("ternary function")
         {
-            SCOPED_TRACE("ternary function");
             auto ffma = xt::fma(a, a, a);
             using assign_traits = xassign_traits<array_type, decltype(ffma)>;
 #if XTENSOR_USE_XSIMD
@@ -915,4 +915,27 @@ namespace xt
 
         EXPECT_EQ(expected, xt::cov(x, y));
     }
+
+
+    TEST(xmath, convolve_full)
+    {
+        xt::xarray<double> x = { 1.0, 3.0, 1.0 };
+        xt::xarray<double> y = { 1.0, 1.0, 1.0 };
+        xt::xarray<double> expected = { 1, 4, 5, 4, 1 };
+
+        auto result = xt::convolve(x, y, xt::convolve_mode::full());
+
+        EXPECT_EQ(result, expected);
+    }
+
+    TEST(xmath, convolve_valid)
+    {
+        xt::xarray<double> x = { 3.0, 1.0, 1.0 };
+        xt::xarray<double> y = { 1.0, 1.0, 1.0 };
+        xt::xarray<double> expected = { 5 };
+
+        auto result = xt::convolve(x, y, xt::convolve_mode::valid());
+
+        EXPECT_EQ(result, expected);
+    }
 }
