@@ -6,14 +6,35 @@
 *                                                                          *
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
+#if defined(_MSC_VER) && !defined(__clang__)
+#define VS_SKIP_XFIXED 1
+#endif
+
+// xfixed leads to ICE in debug mode, this provides
+// an easy way to prevent compilation
+#ifndef VS_SKIP_XFIXED
+
+
+#if (_MSC_VER < 1910 && _WIN64) || (_MSC_VER >= 1910 && !defined(DISABLE_VS2017)) || !defined(_MSC_VER)
 
 #include <complex>
 #include <limits>
 
-#include "gtest/gtest.h"
+#include "test_common_macros.hpp"
 #include "xtensor/xfixed.hpp"
 #include "xtensor/xtensor_config.hpp"
 
+// On VS2015, when compiling in x86 mode, alignas(T) leads to C2718
+// when used for a function parameter, even indirectly. This means that
+// we cannot pass parameters whose class is declared with alignas specifier
+// or any type wrapping or inheriting from such a type.
+// The xtensor_fixed class internally uses aligned_array which is declared as
+// alignas(something_different_from_0), hence the workaround.
+#if _MSC_VER < 1910 && !_WIN64
+#define VS_X86_WORKAROUND 1
+#endif
+
+
 template <typename T>
 class alignas(XTENSOR_FIXED_ALIGN) Foo
 {
@@ -50,3 +71,5 @@ namespace xt
     }
 }
 
+#endif
+#endif // VS_SKIP_XFIXED
